package PureMessage::Policy::BATV;

use strict;

use base 'PureMessage::Policy::Plugin';

use PureMessage::Policy::Spam::Config qw(%config);
use PureMessage::BATV;

use constant batv_config_key => 'plugin#batv.batv';

our $VERSION = '5.509';

my %commands = (
    pmx_batv_watermark => \&pmx_batv_watermark,
    pmx_bounce => \&pmx_bounce,
    pmx_batv_strip => \&pmx_batv_strip,
);

sub commands {
    my $self = shift;
    my %cmds = %commands;
    %commands = ();
    return %cmds;
}

my $batv;
my $timestamp;

sub batv {
    if (!$batv) {
        $batv = PureMessage::BATV->new($config{'plugin'}{'batv'}{'batv'});
    }
    return $batv;
}

sub pmx_batv_watermark {
    my ($msg) = @_;

    my $from = $msg->envelope_from;
    $msg->envelope_from(batv->watermark($from)) if defined $from && $from ne '';
    return 1;
}

=item _check_batv

Internal method which checks BATV.  It does not modify the recipient addresses.
=cut

sub _check_batv {
    my ($msg) = @_;

    $msg->{batv_valid} = 0;
    $msg->{batv_invalid} = 0;
    $msg->{batv_none} = 0;
    $msg->{batv_out_of_date} = 0;

    # We need to check each recipient against batv
    my @rcpt = $msg->envelope_recipients;
    for my $i (0 .. $#rcpt) {
        my $r = batv->verify($rcpt[$i]);
        if ($r) {
            $msg->{batv_valid}++;
        } elsif ($rcpt[$i] =~ /^prvs=[0-9]([0-9]{3})[0-9A-Za-z]{6}=/) {
            if (batv->verify($rcpt[$i], 1000)) {
                # watermark is valid, but out of date
                $msg->{batv_out_of_date}++;
            }
            $msg->{batv_invalid}++;
        } else {
            $msg->{batv_none}++;
        }
    }

    # Log batv to message log.  
    # If multiple batv status's exist it's likely a broken message.
    my $attr;
    
    if (@rcpt eq 1) {
        # Single recipients, easy!
        if ($msg->{batv_valid}) {
            $attr = 'valid';
        } elsif ($msg->{batv_out_of_date}) {
            $attr = 'old';
        } elsif ($msg->{batv_invalid}) {
            $attr = 'invalid';
        }
    } else {
        # Multiple recipients
        if ($msg->{batv_valid} == @rcpt) {
            $attr = 'valid';
        } elsif ($msg->{batv_none} != @rcpt) {
            $attr = 'multiple';
        }
        # $attr = undef if no_batv_at_all();
    }

    $msg->set_attr('mark_batv',$attr) if defined $attr;

}

sub pmx_batv_strip {
    my ($msg, $args) = @_;

    _check_batv($msg) unless exists $msg->{batv_valid};

    my $changed = 0;
    my @rcpt = $msg->envelope_recipients;
    for my $i (0 .. $#rcpt) {
        if ($rcpt[$i] =~ s/^prvs=[0-9]{4}[0-9A-Za-z]{6}=//) {
            $changed = 1;
        }
    }
    $msg->envelope_recipients(@rcpt) if $changed;
    return 1;
}

sub pmx_bounce {
    my ($msg, $args, $first_time) = @_;

    pmx_batv_strip($msg, $args) unless exists $msg->{batv_valid};

    my $use_batv = 0;
    my $is_bounce = 0;

    if ($args->tag('batv')) {
        $use_batv = 1;
    }

    # We need to check each recipient against batv, and strip if valid.

    # Does the message have a recipient?
    if ($msg->envelope_from() eq '') {
        $is_bounce = 1;
    } else { # Check spam rules for OOTO_ or BOUNCE_ rule
        eval {
            require PureMessage::Policy::Spam;
            PureMessage::Policy::Spam::probability($msg);
        };

        if (!$@) {
            my $hits = $msg->get_attr('mark_h');

            if (grep /^BOUNCE_/, @$hits) {
                $is_bounce = 1;
            }
        }
    }

    if ($is_bounce) {
        if ($use_batv && $msg->{batv_valid} && !$msg->{batv_invalid}) {
            # We're using BATV, but the watermark was valid, so we allow it.
            return 0;
        }
        # Its a bounce
        return 1;
    }
    return 0;
}

1;
